<?php
/**
 * Contains usuful functions for working/adding/altering/deleteing zones/domains
 *
 * $Id: functions.zones.php 187 2007-05-11 08:23:17Z rbott $
 *
 * @package ZoneAdmin
 * @author Rudolph Bott
 * @version 0.1
 * @copyright Rudolph Bott
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 */

// vim:tabstop=2:expandtab:shiftwidth=2:filetype=php:syntax:ruler:

//praktische funktionen fuer zonen

/**
 * translates a given domain-id to its domain name
 *
 * the function looks up a given domain-id in the database
 * and returns its domain-name
 * the second parameter defines the table in which
 * to look, either active or inactive domains
 * be default the function checks active domains
 *
 * in case of errors the function might set
 * ERR_MYSQL_SYNTAX and ERR_INVAL_DOMAIN_ID
 * and return an empty string
 *
 * @param integer $id the domain-id
 * @param boolean $disabled if true, lookup $id in disabled-domains table
 * @return string domain name 
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_domain_from_id($id,$disabled = false) {
  global $mysqlcfg;

  if($disabled) {
    $table = $mysqlcfg["remdomtbl"];
  }
  else {
    $table = $mysqlcfg["domtbl"];
  }

  $query = "SELECT name FROM ".$table." WHERE id='".mysql_real_escape_string($id)."'";
  $result = mysql_query($query);

  if(mysql_errno() > 0) {
    add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
    return '';
  }

  if(mysql_num_rows($result) != 1) {
    add_error(ERR_INVAL_DOMAIN_ID,__FUNCTION__);
    return '';
  }

  $cur_zone = mysql_fetch_assoc($result);
  mysql_free_result($result);

  return $cur_zone['name'];
}

/**
 * translates a given domain name to its domain-id
 *
 * the function looks up a given domain-id in the database
 * and returns its matching domain-id
 *
 * in case of errors the function might set ERR_MYSQL_SYNTAX
 * or ERR_INVAL_DOMAIN and return zero
 *
 * @param string $domain the domain name
 * @param boolean $disabled if true, lookup $domain in disabled-domains table
 * @return integer domain-id or zero
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function get_id_from_domain($domain,$disabled = false) {
  global $mysqlcfg;

  if($disabled) {
    $table = $mysqlcfg["remdomtbl"];
  }
  else {
    $table = $mysqlcfg["domtbl"];
  }

  $query = "SELECT id FROM ".$table." WHERE name='".mysql_real_escape_string($domain)."'";
  $result = mysql_query($query);
  
  if(mysql_errno() > 0)  {
    add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
    return 0;
  }

  if(mysql_num_rows($result) > 0) {
    $id = mysql_fetch_assoc($result);
    mysql_free_result($result);
    $result = $id["id"];
//    mysql_free_result($result);
  }
  else {
    add_error(ERR_INVAL_DOMAIN,__FUNCTION__);
    $result = 0;
  }
  
  return $result;
}

/**
 * checks if a given domain exists
 *
 * this function takes a domain name OR a domain id
 * as first parameter and checks if it currently exists
 * or not
 * the second optional parameter determines wether
 * to look the domain up in the active or inactive
 * domains table
 * @param mixed $domain the domain name or domain id
 * @param boolealn $disabled determine in which table to lookup up $domain
 * @return boolean true or false
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function domain_exists($domain,$disabled = false) {
  global $mysqlcfg;

  if($disabled) {
    $table = $mysqlcfg["remdomtbl"];
  }
  else {
    $table = $mysqlcfg["domtbl"];
  }


  if(is_numeric($domain)) {
    $query = "SELECT COUNT(id) FROM ".$table." WHERE id='".mysql_real_escape_string($domain)."'";
  }
  else {
    $query = "SELECT COUNT(id) FROM ".$table." WHERE name='".mysql_real_escape_string($domain)."'";
  }

  $result = mysql_query($query);
  if(mysql_errno() > 0) {
    add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
    return true;
  }
  else {
    if($row = mysql_fetch_row($result)) {
      if($row[0] > 0) {
        return true; //exists
      }
      else {
        return false; //does not exist
      }
    }
    else {
      //mysql_fetch_rows failed...mysql error?!
      add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
      return false;
    }
  }
}

/**
 * returns type of given zone
 *
 * this function returns the type of
 * the given zone (either numeric or domain name)
 * currently, this might be NATIVE, SLAVE or MASTER
 * in capital letters
 * in case of an error or non-existing zone the
 * function throws the respective error and returns UNKONWN
 *
 * @param mixed $domain the domain name or domain id
 * @return string NATIVE, MASTER, SLAVE or UNKNOWN
 * @author Rudolph Bott
 * @version V1.0
*/
function get_domain_type($domain) {
  global $mysqlcfg;

  if(is_numeric($domain)) {
    $query = "SELECT `type` FROM ".$mysqlcfg["domtbl"]." WHERE id='".mysql_real_escape_string($domain)."'";
  }
  else {
    $query = "SELECT `type` FROM ".$mysqlcfg["domtbl"]." WHERE name='".mysql_real_escape_string($domain)."'";
  }

  $result = mysql_query($query);
  if(mysql_errno() > 0) {
    add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
    return 'UNKNOWN';
  }
  else {
    if(mysql_num_rows($result) > 0) {
      $row = mysql_fetch_row($result);
      return strtoupper($row[0]);
    }
    else {
      add_error(ERR_INVAL_DOMAIN,__FUNCTION__);
      return 'UNKNOWN';
    }
  }
  
}



/**
 * validates a given domain *NOT IMPLEMENTED YET*
 *
 * this function checks a given domain name if it
 * appears to be valid (special chars etc.)
 * @param string $domain the domain name
 * @return boolean true or false
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function is_valid_domain($domain) {
  return false;
}


/**
 * retrieves the last change of a domain
 *
 * this function retrieves the date of the last change
 * made to a record in the given zone (date format YYYYMMDD)
 *
 * in case of an error the function sets ERR_MYSQL_SYNTAX
 * and returns "00000000"
 *
 * @param integer $id the domain-id
 * @return integer date as YYYYMMDD
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function last_change_domain($id) {
  $date = "00000000";
  global $mysqlcfg;

  if(!empty($id) && is_numeric($id)) {
    $query = "SELECT MAX(change_date) AS change_date FROM ".$mysqlcfg["rectbl"]." WHERE domain_id='".mysql_real_escape_string($id)."'";
    $result = mysql_query($query);
    if(mysql_errno() > 0) {
      add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
      return $date;
    }
    $row = mysql_fetch_assoc($result);
    $date = $row["change_date"];
  }
  return $date;
}


/**
 * adds a new domain with template support
 *
 * function adds a new domain to the database
 * and uses the given template-id to populate the
 * zone with records defined in the template
 * if the third parameter is set to true (defaults to false)
 * the new domain is created in the disabled-domains table
 * @param string $name domain name
 * @param integer $tempate tempate-id
 * @param boolean $disabled if true, create domain as initially inactive
 * @return integer primary key of mysql insert operation
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function add_domain($name,$template,$disabled = false) {
  global $mysqlcfg;
  global $cfg;

  $tstamp = date("Ymd");

  if($disabled) {
    $dom_tbl = $mysqlcfg["remdomtbl"];
    $rec_tbl = $mysqlcfg["remrectbl"];
  }
  else {
    $dom_tbl = $mysqlcfg["domtbl"];
    $rec_tbl = $mysqlcfg["rectbl"];
  }

  if(!empty($name) && !empty($template) && is_numeric($template)) {
    //neue domain anlegen
    $query = "INSERT INTO ".$dom_tbl." (name,type) VALUES ('".mysql_real_escape_string($name)."','MASTER')";
    mysql_query($query);
    $id = mysql_insert_id();

    add_history($cfg["user"],"ADD ZONE",$name);

    //war anlegen erfolgreich & hat benutzer ein template gewaehlt?
    if($id > 0 && $template > 0) {
      //jeden record aus dem angegebenen template holen, platzhalter ersetzen und als "echte" records anlegen
      $query = "SELECT name,type,prio,content,ttl FROM ".$mysqlcfg["tplrectbl"]." WHERE template_id='".mysql_real_escape_string($template)."'";
      $result = mysql_query($query);
      while($row = mysql_fetch_assoc($result)) {
        $row["name"] = str_replace("[DOMAIN]",$name,$row["name"]);
        $row["content"] = str_replace("[DOMAIN]",$name,$row["content"]);
        $row["content"] = str_replace("[SERIAL]",$tstamp."00",$row["content"]);

        $query = "INSERT INTO ".$rec_tbl." (domain_id,name,type,prio,content,ttl,change_date) VALUES('".$id."','".$row["name"]."','".$row["type"]."','".$row["prio"]."','".$row["content"]."','".$row["ttl"]."','".$tstamp."')";
        mysql_query($query);
      }
      mysql_free_result($result);
    }
    return $id;
  }
  else {
    return false;
  }
}

//eine domain mitsammt ihren records anhand ihrer ID loeschen
/**
 * deletes a domain and its records
 *
 * function to delete a domain and all associated records
 * before actually deleting the domain, all record and zone
 * information are saved to a history entry in serialized form
 * to enable later restores
 *
 * the function might set and exit with ERR_MYSQL_SYNTAX or
 * ERR_INVAL_DOMAIN_ID
 * if the function succeeds, it returns ERR_NOERROR
 *
 * @param integer $id domain-id
 * @param boolean $disabled does $id point to a disabled domain?
 * @return integer error code (ERR_NOERROR = OK)
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function del_domain($id,$disabled = false) {
  global $mysqlcfg;
  global $cfg;

  if(!empty($id) && is_numeric($id) && domain_exists($id)) {
    $domain = get_domain_from_id($id,$disabled);
    $history = get_domain_records($id,$disabled);
    add_history($cfg["user"],"DEL ZONE",$domain,$history);

    if($disabled) {
      $domtbl = $mysqlcfg["remdomtbl"];
      $cmttbl = $mysqlcfg["remcmttbl"];
    }
    else {
      $domtbl = $mysqlcfg["domtbl"];
      $cmttbl = $mysqlcfg["cmttbl"];
    }

    $query = "DELETE FROM ".$domtbl." WHERE id='".mysql_real_escape_string($id)."' LIMIT 1";
    mysql_query($query);
    if(mysql_errno() > 0) {
      add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
      return ERR_MYSQL_SYNTAX;
    }

    //workaround needed because comment tables are not yet working with
    //foreign keys
    $query = "DELETE FROM ".$cmttbl." WHERE domain_id='".mysql_real_escape_string($id)."' LIMIT 1";
    mysql_query($query);
    if(mysql_errno() > 0) {
      add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
      return ERR_MYSQL_SYNTAX;
    }

    //if we're still "in" the function, everything must have worked out fine..so we exit cleanly
    return ERR_NOERROR;
  }
  else {
    add_error(ERR_INVAL_DOMAIN_ID,__FUNCTION__);
    return ERR_INVAL_DOMAIN_ID;
  }
}

/**
 * disables a domain with its records
 *
 * function to disable a domain and all associated records
 * this is achieved by moving it to different domain/record tables
 *
 * the function might set and return ERR_MYSQL_SYNTAX or
 * ERR_INVAL_DOMAIN_ID
 * if it succeeds it will return ERR_NOERROR
 *
 * @param integer $id domain-id
 * @return integer error code (ERR_NOERROR = OK)
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function disable_domain($id) {
  global $mysqlcfg;
  global $cfg;
  $error = false;

  if(!empty($id) && is_numeric($id)) {
    if(domain_exists($id)) {
      $saveorigin = $cfg["striporigin"];
      $cfg["striporigin"] = false;
      $domain = get_domain_from_id($id);
      $domain_comment = get_comment($id,0);
      $records = get_domain_records($id);
      $cfg["striporigin"] = $saveorigin;

      $query = "INSERT INTO ".$mysqlcfg["remdomtbl"]." (name,type) VALUES('".$domain."','NATIVE')";
      mysql_query($query);

      if(mysql_errno() > 0) {
        add_error(ERR_MYSQL_SYNTAX,__FUNCTION__);
        return ERR_MYSQL_SYNTAX;
      }

      add_history($cfg["user"],"DISABLED ZONE",$domain);

      $newid = mysql_insert_id();
      $newid = 5;
      if($id > 0) {
        if(isset($domain_comment["msg"]) && !empty($domain_comment["msg"])) {
          $query = "INSERT INTO ".$mysqlcfg["remcmttbl"]." (`domain_id`,`record_id`,`user`,`when`,`msg`) SELECT id, '0', '".$domain_comment["user"]."', '".$domain_comment["when"]."', '".$domain_comment["msg"]."' FROM ".$mysqlcfg["remdomtbl"]." WHERE name='".$domain."'";
          mysql_query($query);
        }
        foreach($records AS $record) {
          if(!$error) {
            $query = "INSERT INTO ".$mysqlcfg["remrectbl"]." (domain_id,name,type,content,ttl,prio) select id, '".mysql_real_escape_string($record["name"])."', '".mysql_real_escape_string($record["type"])."', '".mysql_real_escape_string($record["content"])."', '".mysql_real_escape_string($record["ttl"])."', '".mysql_real_escape_string($record["prio"])."' from ".$mysqlcfg["remdomtbl"]." WHERE name='".$domain."'";
            mysql_query($query);
            if(mysql_errno() > 0) $error = true;
            $rec_id = mysql_insert_id();
            if(!empty($record["comment"])) {
              $query = "INSERT INTO ".$mysqlcfg["remcmttbl"]." (`domain_id`, `record_id`, `user`, `when`, `msg`) select id, '$rec_id', '".$record["comment_user"]."', '".$record["comment_when"]."', '".$record["comment"]."' FROM ".$mysqlcfg["remdomtbl"]." WHERE name='".$domain."'";
              mysql_query($query);
              if(mysql_errno() > 0) $error = true;
            }
          }
        }
      }
      else return false;
      if(!$error) {
        $query = "DELETE FROM ".$mysqlcfg["domtbl"]." WHERE id='".mysql_real_escape_string($id)."' LIMIT 1";
        mysql_query($query);
        $query = "DELETE FROM ".$mysqlcfg["cmttbl"]." WHERE domain_id='".mysql_real_escape_string($id)."'";
        mysql_query($query);
        return ERR_NOERROR;
      }
    }
  }
  else {
    add_error(ERR_INVAL_DOMAIN_ID,__FUNCTION__);
    return ERR_INVAL_DOMAIN_ID;
  }
}

/**
 * enables a domain with its records
 *
 * function to enable a domain and all associated records
 *
 * this is achieved by moving it to back to the original domain/record tables
 *
 * the function might set ERR_INVAL_DOMAIN_ID
 *
 * @param integer $id domain-id
 * @return integer status of last mysql operation (0 = OK)
 * @todo implement better/proper error handling
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function enable_domain($id) {
  global $mysqlcfg;
  global $cfg;
  $error = false;

  //override global striporigin setting because
  //we have to make sure that get_domain_records() returns
  //fully qualified domain names
  $saveorigin = $cfg["striporigin"];
  $cfg["striporigin"] = false;

  if(!empty($id) && is_numeric($id) && domain_exists($id,true)) {
    //get domain name
    $query = "SELECT name FROM ".$mysqlcfg["remdomtbl"]." WHERE id='".mysql_real_escape_string($id)."'";
    $result = mysql_query($query);
    $row = mysql_fetch_assoc($result);
    $domain = $row["name"];
    mysql_free_result($result);

    //check for domain comment
    $query = "SELECT * FROM ".$mysqlcfg["remcmttbl"]." WHERE domain_id='".mysql_real_escape_string($id)."' AND record_id='0'";
    $result = mysql_query($query);
    if(mysql_num_rows($result) > 0) $domain_comment = mysql_fetch_assoc($result);
    mysql_free_result($result);

    //get domain records + comments
    $records = get_domain_records($id,true);

    //get domain comments
    $query = "SELECT * FROM ".$mysqlcfg["remcmttbl"]." WHERE domain_id='".mysql_real_escape_string($id)."' AND record_id > 0";
    $result = mysql_query($query);
    while($row = mysql_fetch_assoc($result)) {
      $comments[] = $row;
    }
    mysql_free_result($result);
    
    add_history($cfg["user"],"ENABLED ZONE",$domain);

    $query = "INSERT INTO ".$mysqlcfg["domtbl"]." (name,type) VALUES('".$domain."','MASTER')";
    mysql_query($query);
    $newid = mysql_insert_id();

    if(isset($domain_comment)) {
      $query = "INSERT INTO ".$mysqlcfg["cmttbl"]." (`domain_id`, `record_id`, `user`, `when`, `msg`) SELECT id, '0', '".$domain_comment["user"]."', '".$domain_comment["when"]."', '".$domain_comment["msg"]."' FROM ".$mysqlcfg["domtbl"]." WHERE name='".$domain."'";
      mysql_query($query);

      $query = "DELETE FROM ".$mysqlcfg["remcmttbl"]." WHERE domain_id='".mysql_real_escape_string($id)."' AND record_id='0' LIMIT 1";
      mysql_query($query);
    }


    foreach($records AS $record) {
      if(!$error) {
        $query = "INSERT INTO ".$mysqlcfg["rectbl"]." (domain_id,name,type,content,ttl,prio) select id, '".mysql_real_escape_string($record["name"])."', '".mysql_real_escape_string($record["type"])."', '".mysql_real_escape_string($record["content"])."', '".mysql_real_escape_string($record["ttl"])."', '".mysql_real_escape_string($record["prio"])."' FROM ".$mysqlcfg["domtbl"]." WHERE name='".$domain."'";
        mysql_query($query);
        $rec_id = mysql_insert_id();
        if(mysql_errno() > 0) $error = true;
        if(!empty($record["comment"])) {
          $query = "INSERT INTO ".$mysqlcfg["cmttbl"]." (`domain_id`, `record_id`, `user`, `when`, `msg`) SELECT id, $rec_id, '".$record["comment_user"]."', '".$record["comment_when"]."', '".$record["comment"]."' FROM ".$mysqlcfg["domtbl"]." WHERE name='".$domain."'";
          mysql_query($query);
        }
      }
    }
    if(!$error) {
      $query = "DELETE FROM ".$mysqlcfg["remdomtbl"]." WHERE id='".mysql_real_escape_string($id)."' LIMIT 1";
      mysql_query($query);
      $query = "DELETE FROM ".$mysqlcfg["remcmttbl"]." WHERE domain_id='".mysql_real_escape_string($id)."'";
      $cfg["striporigin"] = $saveorigin;
      return true;
    }
  }
  else {
    $cfg["striporigin"] = $saveorigin;
    add_error(ERR_INVAL_DOMAIN_ID,__FUNCTION__);
    return ERR_INVAL_DOMAIN_ID;
  }
}


/**
 * checks if a given date (YYYYMMDD) is valid
 *
 * splits the date into its three parts and checks
 * they contain valid information
 * @param integer $date YYYYMMMDD date
 * @return boolean true or false
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function check_date($date) {
  $result = false;

  if(strlen($date) == 8) {
    $year = substr($date,0,4);
    $month = substr($date,4,2);
    $day = substr($date,-2);
    $result = checkdate($month,$day,$year);
  }

  return $result;
}

/**
 * parses a SOA record from a given domain
 *
 * function retrieves the SOA from a given domain-id
 * and parses it into an associative array (keys: master, contact, serial, refresh, retry, expire, ttl)
 *
 * the function might set ERR_INVAL_SOA, ERR_MISSING_SOA or
 * ERR_INVAL_DOMAIN_ID
 *
 * @param integer $id domain-id
 * @return array associative array
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function parse_soa($id) {
  global $mysqlcfg;
  global $cfg;

  $soa = array("master" => "", "contact" => "", "serial" => 0, "refresh" => 0, "retry" => 0, "expire" => 0, "ttl" => 0);

  if(!empty($id) && is_numeric($id) && domain_exists($id)) {
    $record = get_domain_records_by_type($id,"SOA");
    if(count($record) == 1) {
      $split = explode(" ",trim($record[0]["content"]));
      if(count($split) == 7) {
        $soa["master"] = $split[0];
        $soa["contact"] = $split[1];
        $soa["serial"] = $split[2];
        $soa["refresh"] = $split[3];
        $soa["retry"] = $split[4];
        $soa["expire"] = $split[5];
        $soa["ttl"] = $split[6];
        $soa["record-id"] = $record[0]["id"];
      }
      else {
        add_error(ERR_INVAL_SOA,__FUNCTION__);
      }
    }
    else if (count($record) == 0) {
      add_error(ERR_MISSING_SOA,__FUNCTION__);
    }
  }
  else {
    add_error(ERR_INVAL_DOMAIN_ID,__FUNCTION__);
  }

  return $soa;
}

/**
 * raises the serial for a given domain
 *
 * function to fetch current serial from a given domain and
 * raise it
 *
 * the function might set and return ERR_INVAL_SERIAL
 * or ERR_MISSING_SOA
 *
 * @param integer $id domain-id
 * @return integer value of raised serial or error code
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/
function raise_serial($id) {
  global $mysqlcfg;
  global $cfg;

  $newserial = 0;
  $today = date("Ymd",time());

  if(!empty($id) && is_numeric($id)) {
    $soa = get_domain_records_by_type($id,"SOA");
    if(count($soa) == 1) {
      list($server,$contact,$serial,$refresh,$retry,$expire,$ttl) = explode(" ",$soa[0]["content"]);
      if(strlen($serial) == 10) {
        $date = substr($serial,0,8);
        $counter = substr($serial,-2);
        if(check_date($date)) {
          if($date == $today && $counter < 99) {
            $counter = $counter+1;
            if(strlen($counter) == 1) $counter = "0".$counter;
            $newserial = $date.$counter;
          }
          else {
            $newserial = $today."00";
          }
          $newSOA = $server." ".$contact." ".$newserial." ".$refresh." ".$retry." ".$expire." ".$ttl;
          update_domain_record($soa[0]["domain_id"],$soa[0]["id"],$soa[0]["name"],"SOA",$newSOA,0);
        }
      }
      else {
        add_error(ERR_INVAL_SERIAL,__FUNCTION__);
        return ERR_INVAL_SERIAL;
      }
    }
    else if(count($soa) == 0) {
      add_error(ERR_MISSING_SOA,__FUNCTION__);
    }
  }
  return $newserial;
}


/**
 * returns a given zone in zonefile-format
 *
 * function to dump a given zone in zonefile-format
 * (bind-style)
 *
 * if the second optional parameter is set to true
 * retrieve zone from disabled-zones table instead
 * of the regular zones table
 * @param integer $id domain-id
 * @param boolean $disabled if true, retrieve zone from disabled-zones table
 * @return string zonefile
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/

function dump_to_zonefile($id,$disabled = false) {
  $result = "";

  //fetch domain records
  $records = get_domain_records($id);

  foreach($records AS $record) {
    //if the current record happens to be either NS, MX or CNAME, add the
    //final "." to the hostname (required by bind)
    if($record["type"] == "NS" || $record["type"] == "MX" || $record["type"] == "CNAME") {
      $record["content"] .= ".";
    }

    //SOA and MX need some special output treatment, the rest
    //is handled by the "default" section
    switch($record["type"]) {
      case 'SOA':
        //parse the soa record of the current domain
        $soa = parse_soa($id);

        //add SOA record (including nasty tabstops)
        $result .= "@\t\tIN\tSOA\t".$soa["master"].". ".$soa["contact"].". (\n\t\t\t\t".$soa["serial"]."\n\t\t\t\t".$soa["refresh"]."\n\t\t\t\t".$soa["retry"]."\n\t\t\t\t".$soa["expire"]."\n\t\t\t\t".$soa["ttl"]." )\n\n";
      break;
      case 'MX':
        //if someone attached a comment to this record,
        //output it now
        //additionally prefix all newlines with a ; so bind
        //won't complain
        if(!empty($record["comment"])) {
          $result .= "\n;".str_replace("\n","\n;",trim($record["comment"]))."\n";
        }

        //output the MX record including priority
        $result .= $record["name"]."\t\tIN\t".$record["type"]." ".$record["prio"]."\t".$record["content"]."\n";
        break;
      break;
      default:
        //see above comment (same code)
        if(!empty($record["comment"])) {
          $result .= "\n;".str_replace("\n","\n;",trim($record["comment"]))."\n";
        }

        //if the current record's name is longer then 7 chars, only
        //use one tabstop
        //if it's shorter, use two of them (just for nicer output)
        if(strlen($record["name"]) > 7) {
          $tab = "\t";
        }
        else {
          $tab = "\t\t";
        }

        //output the record
        $result .= $record["name"].$tab."IN\t".$record["type"]."\t".$record["content"]."\n";
        break;
    }
  }
  return $result;
}

/**
 * returns a given zone as SQL dump
 *
 * function to dump a given zone as
 * SQL statements (excluding comments)
 *
 * @param integer $id domain-id
 * @return string SQL dump
 * @author Rudolph Bott
 * @copyright Copyright 2007, Rudolph Bott
 * @version V1.0
*/

function dump_to_sql($id) {
  global $cfg;

  $result = "";

  //override global striporigin setting because
  //we have to make sure that get_domain_records() returns
  //fully qualified domain names
  $saveorigin = $cfg["striporigin"];
  $cfg["striporigin"] = false;

  //fetch domain name
  $domain = get_domain_from_id($id);

  $result .= "INSERT INTO domains (name,type) VALUES('".$domain."','MASTER');\n";

  //fetch records
  $records = get_domain_records($id);

  //process records
  //as of now, this is a plain powerdns-sql-dump
  //no ZoneAdmin-specific information (e.g. comments)
  //are included here
  foreach($records AS $record) {
    $result .= "INSERT INTO records (domain_id,name,type,content,ttl,prio) select id , '".$record["name"]."', '".$record["type"]."', '".$record["content"]."', ".$record["ttl"].", ".$record["prio"]." from domains where name='".$domain."';\n";
  }

  //re-set the striporigin option to its original value
  $cfg["striporigin"] = $saveorigin;

  return $result;
}

?>
